function [ match_idx, epi_1, match_err] = searchEpipolar( I0_padded, I1_padded, px_0, half_win_size, methods )
%SEARCHEPIPOLAR search the epipolar line corresponding to px_0 and return
%               the best match index, using different error measurements
%   NOTE: px_0, epi_1 and px_1 are both of the coordinates for unpadded image
%         coordinates in the padded images are only used internally.
%         EXERCISE5: add a D_range parameter to limit the search range
%                    D_range 2x1 search disparity range
%   input:
%       I0_padded,I1_padded: images padded by half_win_size, note that
%                            these images should be rectified and
%       px_0: pixel to match in I0
%       half_win_size: half the size of the window used for match
%       match_methods: cell of strings, possible values are 'SSD', 'SAD',
%                      'ZNCC'.
%   output:
%       match_idx: k x 1, best matched pixel index on epi_1 for corresponding 
%                  error measuremnts
%       epi_1: 2 x n, epipolar line in I1
%       match_err: k x n, matrix storing the similarity score,
%                         where k stands for the number of methods input

% check
methods_num = size(methods,2);
for i=1:methods_num
    element = cell2mat(methods(i));
    assert(isequal(element, 'SSD') ||...
        isequal(element, 'SAD') || ...
        isequal(element, 'ZNCC'));
end

win_size = 2*half_win_size+1;

% calculate the epipolar line
img_size = flipud(size(I0_padded)' - 2*half_win_size);
epi_1 = rectifiedEpipolarLine(px_0, img_size); % IMPLEMENT
if isempty(epi_1)
    match_idx = [];
    match_err = [];
    return;
end

% get image patch from I0_padded and I1_padded
N = size(epi_1, 2);
px_0 = px_0 + half_win_size;
patch_0 = I0_padded(px_0(2)-half_win_size:px_0(2)+half_win_size, ...
    px_0(1)-half_win_size:px_0(1)+half_win_size);
patch_0 = double(patch_0);
patch_1 = zeros(win_size, N*win_size);
for i=1:N
    ca_1 = epi_1(:, i); % candidate
    ca_1 = ca_1 + half_win_size;
    patch_1(:, (i-1)*win_size+1:i*win_size) = ...
        I1_padded(ca_1(2)-half_win_size:ca_1(2)+half_win_size,...
        ca_1(1)-half_win_size:ca_1(1)+half_win_size);
end

% calculate error
match_err = zeros(methods_num, N);
match_idx = zeros(methods_num, 1);
for i=1:methods_num
    method = cell2mat(methods(i));
    for j=1:N
        if isequal(method, 'SSD')
            match_err(i,j) = SSD(patch_0, patch_1(:,(j-1)*win_size+1:j*win_size)); % IMPLEMENT
        elseif isequal(method, 'SAD')
            match_err(i,j) = SAD(patch_0, patch_1(:,(j-1)*win_size+1:j*win_size)); % IMPLEMENT
        elseif isequal(method, 'ZNCC')
            match_err(i,j) = ZNCC(patch_0, patch_1(:,(j-1)*win_size+1:j*win_size)); % IMPLMENT
        else
            assert(false, 'unknown error metric');
        end
    end
    
    % EXERCISE 2: find the best match according to the similarity score
    %            Note the difference among similarity measurements
    best_match = 1;
    if isequal(method, 'SSD') || isequal(method, 'SAD')
        % find best match for SSD and SAD
	elseif isequal(method, 'ZNCC')
        % find best match for ZNCC
    else
        assert(false, 'unknown error metric');
    end
    match_idx(i) = best_match;
end

end

